@@ -251,12 +251,10 @@
 #include "llvm/ADT/SmallSet.h"
 #include "llvm/Support/StringPool.h"
 #include "llvm/Support/raw_ostream.h"
+#include "ModularizeUtilities.h"
 
 namespace Modularize {
 
-// Forwards.
-class PreprocessorTrackerImpl;
-
 // Some handle types
 typedef llvm::PooledStringPtr StringHandle;
 
@@ -302,7 +300,8 @@ static void getSourceLocationLineAndColumn(clang::Preprocessor &PP,
 }
 
 // Retrieve source snippet from file image.
-std::string getSourceString(clang::Preprocessor &PP, clang::SourceRange Range) {
+static std::string getSourceString(clang::Preprocessor &PP,
+                                   clang::SourceRange Range) {
   clang::SourceLocation BeginLoc = Range.getBegin();
   clang::SourceLocation EndLoc = Range.getEnd();
   const char *BeginPtr = PP.getSourceManager().getCharacterData(BeginLoc);
@@ -312,7 +311,8 @@ std::string getSourceString(clang::Preprocessor &PP, clang::SourceRange Range) {
 }
 
 // Retrieve source line from file image given a location.
-std::string getSourceLine(clang::Preprocessor &PP, clang::SourceLocation Loc) {
+static std::string getSourceLine(clang::Preprocessor &PP,
+                                 clang::SourceLocation Loc) {
   const llvm::MemoryBuffer *MemBuffer =
       PP.getSourceManager().getBuffer(PP.getSourceManager().getFileID(Loc));
   const char *Buffer = MemBuffer->getBufferStart();
@@ -337,8 +337,8 @@ std::string getSourceLine(clang::Preprocessor &PP, clang::SourceLocation Loc) {
 }
 
 // Retrieve source line from file image given a file ID and line number.
-std::string getSourceLine(clang::Preprocessor &PP, clang::FileID FileID,
-                          int Line) {
+static std::string getSourceLine(clang::Preprocessor &PP, clang::FileID FileID,
+                                 int Line) {
   const llvm::MemoryBuffer *MemBuffer = PP.getSourceManager().getBuffer(FileID);
   const char *Buffer = MemBuffer->getBufferStart();
   const char *BufferEnd = MemBuffer->getBufferEnd();
@@ -374,10 +374,10 @@ std::string getSourceLine(clang::Preprocessor &PP, clang::FileID FileID,
 // for the macro instance, which in the case of a function-style
 // macro will be a ')', but for an object-style macro, it
 // will be the macro name itself.
-std::string getMacroUnexpandedString(clang::SourceRange Range,
-                                     clang::Preprocessor &PP,
-                                     llvm::StringRef MacroName,
-                                     const clang::MacroInfo *MI) {
+static std::string getMacroUnexpandedString(clang::SourceRange Range,
+                                            clang::Preprocessor &PP,
+                                            llvm::StringRef MacroName,
+                                            const clang::MacroInfo *MI) {
   clang::SourceLocation BeginLoc(Range.getBegin());
   const char *BeginPtr = PP.getSourceManager().getCharacterData(BeginLoc);
   size_t Length;
@@ -399,26 +399,25 @@ std::string getMacroUnexpandedString(clang::SourceRange Range,
 // allows modularize to effectively work with respect to macro
 // consistency checking, although it displays the incorrect
 // expansion in error messages.
-std::string getMacroExpandedString(clang::Preprocessor &PP,
-                                   llvm::StringRef MacroName,
-                                   const clang::MacroInfo *MI,
-                                   const clang::MacroArgs *Args) {
+static std::string getMacroExpandedString(clang::Preprocessor &PP,
+                                          llvm::StringRef MacroName,
+                                          const clang::MacroInfo *MI,
+                                          const clang::MacroArgs *Args) {
   std::string Expanded;
   // Walk over the macro Tokens.
-  typedef clang::MacroInfo::tokens_iterator Iter;
-  for (Iter I = MI->tokens_begin(), E = MI->tokens_end(); I != E; ++I) {
-    clang::IdentifierInfo *II = I->getIdentifierInfo();
+  for (const auto &T : MI->tokens()) {
+    clang::IdentifierInfo *II = T.getIdentifierInfo();
     int ArgNo = (II && Args ? MI->getArgumentNum(II) : -1);
     if (ArgNo == -1) {
       // This isn't an argument, just add it.
       if (II == nullptr)
-        Expanded += PP.getSpelling((*I)); // Not an identifier.
+        Expanded += PP.getSpelling(T); // Not an identifier.
       else {
         // Token is for an identifier.
         std::string Name = II->getName().str();
         // Check for nexted macro references.
         clang::MacroInfo *MacroInfo = PP.getMacroInfo(II);
-        if (MacroInfo)
+        if (MacroInfo && (Name != MacroName))
           Expanded += getMacroExpandedString(PP, Name, MacroInfo, nullptr);
         else
           Expanded += Name;
@@ -457,77 +456,7 @@ std::string getMacroExpandedString(clang::Preprocessor &PP,
   return Expanded;
 }
 
-// Get the string representing a vector of Tokens.
-std::string
-getTokensSpellingString(clang::Preprocessor &PP,
-                        llvm::SmallVectorImpl<clang::Token> &Tokens) {
-  std::string Expanded;
-  // Walk over the macro Tokens.
-  typedef llvm::SmallVectorImpl<clang::Token>::iterator Iter;
-  for (Iter I = Tokens.begin(), E = Tokens.end(); I != E; ++I)
-    Expanded += PP.getSpelling(*I); // Not an identifier.
-  return llvm::StringRef(Expanded).trim().str();
-}
-
-// Get the expansion for a macro instance, given the information
-// provided by PPCallbacks.
-std::string getExpandedString(clang::Preprocessor &PP,
-                              llvm::StringRef MacroName,
-                              const clang::MacroInfo *MI,
-                              const clang::MacroArgs *Args) {
-  std::string Expanded;
-  // Walk over the macro Tokens.
-  typedef clang::MacroInfo::tokens_iterator Iter;
-  for (Iter I = MI->tokens_begin(), E = MI->tokens_end(); I != E; ++I) {
-    clang::IdentifierInfo *II = I->getIdentifierInfo();
-    int ArgNo = (II && Args ? MI->getArgumentNum(II) : -1);
-    if (ArgNo == -1) {
-      // This isn't an argument, just add it.
-      if (II == nullptr)
-        Expanded += PP.getSpelling((*I)); // Not an identifier.
-      else {
-        // Token is for an identifier.
-        std::string Name = II->getName().str();
-        // Check for nexted macro references.
-        clang::MacroInfo *MacroInfo = PP.getMacroInfo(II);
-        if (MacroInfo)
-          Expanded += getMacroExpandedString(PP, Name, MacroInfo, nullptr);
-        else
-          Expanded += Name;
-      }
-      continue;
-    }
-    // We get here if it's a function-style macro with arguments.
-    const clang::Token *ResultArgToks;
-    const clang::Token *ArgTok = Args->getUnexpArgument(ArgNo);
-    if (Args->ArgNeedsPreexpansion(ArgTok, PP))
-      ResultArgToks = &(const_cast<clang::MacroArgs *>(Args))
-          ->getPreExpArgument(ArgNo, MI, PP)[0];
-    else
-      ResultArgToks = ArgTok; // Use non-preexpanded Tokens.
-    // If the arg token didn't expand into anything, ignore it.
-    if (ResultArgToks->is(clang::tok::eof))
-      continue;
-    unsigned NumToks = clang::MacroArgs::getArgLength(ResultArgToks);
-    // Append the resulting argument expansions.
-    for (unsigned ArgumentIndex = 0; ArgumentIndex < NumToks; ++ArgumentIndex) {
-      const clang::Token &AT = ResultArgToks[ArgumentIndex];
-      clang::IdentifierInfo *II = AT.getIdentifierInfo();
-      if (II == nullptr)
-        Expanded += PP.getSpelling(AT); // Not an identifier.
-      else {
-        // It's an identifier.  Check for further expansion.
-        std::string Name = II->getName().str();
-        clang::MacroInfo *MacroInfo = PP.getMacroInfo(II);
-        if (MacroInfo)
-          Expanded += getMacroExpandedString(PP, Name, MacroInfo, nullptr);
-        else
-          Expanded += Name;
-      }
-    }
-  }
-  return Expanded;
-}
+namespace {
 
 // ConditionValueKind strings.
 const char *
@@ -804,6 +733,8 @@ public:
   std::vector<ConditionalExpansionInstance> ConditionalExpansionInstances;
 };
 
+class PreprocessorTrackerImpl;
+
 // Preprocessor callbacks for modularize.
 //
 // This class derives from the Clang PPCallbacks class to track preprocessor
@@ -815,7 +746,7 @@ public:
   PreprocessorCallbacks(PreprocessorTrackerImpl &ppTracker,
                         clang::Preprocessor &PP, llvm::StringRef rootHeaderFile)
       : PPTracker(ppTracker), PP(PP), RootHeaderFile(rootHeaderFile) {}
-  ~PreprocessorCallbacks() {}
+  ~PreprocessorCallbacks() override {}
 
   // Overridden handlers.
   void InclusionDirective(clang::SourceLocation HashLoc,
@@ -825,24 +756,26 @@ public:
                           const clang::FileEntry *File,
                           llvm::StringRef SearchPath,
                           llvm::StringRef RelativePath,
-                          const clang::Module *Imported);
+                          const clang::Module *Imported) override;
   void FileChanged(clang::SourceLocation Loc,
                    clang::PPCallbacks::FileChangeReason Reason,
                    clang::SrcMgr::CharacteristicKind FileType,
-                   clang::FileID PrevFID = clang::FileID());
+                   clang::FileID PrevFID = clang::FileID()) override;
   void MacroExpands(const clang::Token &MacroNameTok,
-                    const clang::MacroDirective *MD, clang::SourceRange Range,
-                    const clang::MacroArgs *Args);
+                    const clang::MacroDefinition &MD, clang::SourceRange Range,
+                    const clang::MacroArgs *Args) override;
   void Defined(const clang::Token &MacroNameTok,
-               const clang::MacroDirective *MD, clang::SourceRange Range);
+               const clang::MacroDefinition &MD,
+               clang::SourceRange Range) override;
   void If(clang::SourceLocation Loc, clang::SourceRange ConditionRange,
-          clang::PPCallbacks::ConditionValueKind ConditionResult);
+          clang::PPCallbacks::ConditionValueKind ConditionResult) override;
   void Elif(clang::SourceLocation Loc, clang::SourceRange ConditionRange,
-            clang::PPCallbacks::ConditionValueKind ConditionResult, clang::SourceLocation IfLoc);
+            clang::PPCallbacks::ConditionValueKind ConditionResult,
+            clang::SourceLocation IfLoc) override;
   void Ifdef(clang::SourceLocation Loc, const clang::Token &MacroNameTok,
-             const clang::MacroDirective *MD);
+             const clang::MacroDefinition &MD) override;
   void Ifndef(clang::SourceLocation Loc, const clang::Token &MacroNameTok,
-              const clang::MacroDirective *MD);
+              const clang::MacroDefinition &MD) override;
 
 private:
   PreprocessorTrackerImpl &PPTracker;
@@ -866,14 +799,24 @@ ConditionalExpansionMapIter;
 // course of running modularize.
 class PreprocessorTrackerImpl : public PreprocessorTracker {
 public:
-  PreprocessorTrackerImpl()
-      : CurrentInclusionPathHandle(InclusionPathHandleInvalid),
-        InNestedHeader(false) {}
-  ~PreprocessorTrackerImpl() {}
+  PreprocessorTrackerImpl(llvm::SmallVector<std::string, 32> &Headers,
+        bool DoBlockCheckHeaderListOnly)
+      : BlockCheckHeaderListOnly(DoBlockCheckHeaderListOnly),
+        CurrentInclusionPathHandle(InclusionPathHandleInvalid),
+        InNestedHeader(false) {
+    // Use canonical header path representation.
+    for (llvm::ArrayRef<std::string>::iterator I = Headers.begin(),
+      E = Headers.end();
+      I != E; ++I) {
+      HeaderList.push_back(getCanonicalPath(*I));
+    }
+  }
+
+  ~PreprocessorTrackerImpl() override {}
 
   // Handle entering a preprocessing session.
   void handlePreprocessorEntry(clang::Preprocessor &PP,
-                               llvm::StringRef rootHeaderFile) {
+                               llvm::StringRef rootHeaderFile) override {
     HeadersInThisCompile.clear();
     assert((HeaderStack.size() == 0) && "Header stack should be empty.");
     pushHeaderHandle(addHeader(rootHeaderFile));
@@ -881,14 +824,19 @@ public:
                                                                rootHeaderFile));
   }
   // Handle exiting a preprocessing session.
-  void handlePreprocessorExit() { HeaderStack.clear(); }
+  void handlePreprocessorExit() override { HeaderStack.clear(); }
 
   // Handle include directive.
   // This function is called every time an include directive is seen by the
   // preprocessor, for the purpose of later checking for 'extern "" {}' or
   // "namespace {}" blocks containing #include directives.
   void handleIncludeDirective(llvm::StringRef DirectivePath, int DirectiveLine,
-                              int DirectiveColumn, llvm::StringRef TargetPath) {
+                              int DirectiveColumn,
+                              llvm::StringRef TargetPath) override {
+    // If it's not a header in the header list, ignore it with respect to
+    // the check.
+    if (BlockCheckHeaderListOnly && !isHeaderListHeader(TargetPath))
+      return;
     HeaderHandle CurrentHeaderHandle = findHeaderHandle(DirectivePath);
     StringHandle IncludeHeaderHandle = addString(TargetPath);
     for (std::vector<PPItemKey>::const_iterator I = IncludeDirectives.begin(),
@@ -909,14 +857,17 @@ public:
   bool checkForIncludesInBlock(clang::Preprocessor &PP,
                                clang::SourceRange BlockSourceRange,
                                const char *BlockIdentifierMessage,
-                               llvm::raw_ostream &OS) {
+                               llvm::raw_ostream &OS) override {
     clang::SourceLocation BlockStartLoc = BlockSourceRange.getBegin();
     clang::SourceLocation BlockEndLoc = BlockSourceRange.getEnd();
     // Use block location to get FileID of both the include directive
     // and block statement.
     clang::FileID FileID = PP.getSourceManager().getFileID(BlockStartLoc);
     std::string SourcePath = getSourceLocationFile(PP, BlockStartLoc);
+    SourcePath = ModularizeUtilities::getCanonicalPath(SourcePath);
     HeaderHandle SourceHandle = findHeaderHandle(SourcePath);
+    if (SourceHandle == -1)
+      return true;
     int BlockStartLine, BlockStartColumn, BlockEndLine, BlockEndColumn;
     bool returnValue = true;
     getSourceLocationLineAndColumn(PP, BlockStartLoc, BlockStartLine,
@@ -959,15 +910,19 @@ public:
     if (!InNestedHeader)
       InNestedHeader = !HeadersInThisCompile.insert(H).second;
   }
+
   // Handle exiting a header source file.
   void handleHeaderExit(llvm::StringRef HeaderPath) {
     // Ignore <built-in> and <command-line> to reduce message clutter.
     if (HeaderPath.startswith("<"))
       return;
     HeaderHandle H = findHeaderHandle(HeaderPath);
+    HeaderHandle TH;
     if (isHeaderHandleInStack(H)) {
-      while ((H != getCurrentHeaderHandle()) && (HeaderStack.size() != 0))
+      do {
+        TH = getCurrentHeaderHandle();
         popHeaderHandle();
+      } while ((TH != H) && (HeaderStack.size() != 0));
     }
     InNestedHeader = false;
   }
@@ -975,11 +930,29 @@ public:
   // Lookup/add string.
   StringHandle addString(llvm::StringRef Str) { return Strings.intern(Str); }
 
+  // Convert to a canonical path.
+  std::string getCanonicalPath(llvm::StringRef path) const {
+    std::string CanonicalPath(path);
+    std::replace(CanonicalPath.begin(), CanonicalPath.end(), '\\', '/');
+    return CanonicalPath;
+  }
+
+  // Return true if the given header is in the header list.
+  bool isHeaderListHeader(llvm::StringRef HeaderPath) const {
+    std::string CanonicalPath = getCanonicalPath(HeaderPath);
+    for (llvm::ArrayRef<std::string>::iterator I = HeaderList.begin(),
+        E = HeaderList.end();
+        I != E; ++I) {
+      if (*I == CanonicalPath)
+        return true;
+    }
+    return false;
+  }
+
   // Get the handle of a header file entry.
   // Return HeaderHandleInvalid if not found.
   HeaderHandle findHeaderHandle(llvm::StringRef HeaderPath) const {
-    std::string CanonicalPath(HeaderPath);
-    std::replace(CanonicalPath.begin(), CanonicalPath.end(), '\\', '/');
+    std::string CanonicalPath = getCanonicalPath(HeaderPath);
     HeaderHandle H = 0;
     for (std::vector<StringHandle>::const_iterator I = HeaderPaths.begin(),
                                                    E = HeaderPaths.end();
@@ -993,8 +966,7 @@ public:
   // Add a new header file entry, or return existing handle.
   // Return the header handle.
   HeaderHandle addHeader(llvm::StringRef HeaderPath) {
-    std::string CanonicalPath(HeaderPath);
-    std::replace(CanonicalPath.begin(), CanonicalPath.end(), '\\', '/');
+    std::string CanonicalPath = getCanonicalPath(HeaderPath);
     HeaderHandle H = findHeaderHandle(CanonicalPath);
     if (H == HeaderHandleInvalid) {
       H = HeaderPaths.size();
@@ -1169,7 +1141,7 @@ public:
 
   // Report on inconsistent macro instances.
   // Returns true if any mismatches.
-  bool reportInconsistentMacros(llvm::raw_ostream &OS) {
+  bool reportInconsistentMacros(llvm::raw_ostream &OS) override {
     bool ReturnValue = false;
     // Walk all the macro expansion trackers in the map.
     for (MacroExpansionMapIter I = MacroExpansions.begin(),
@@ -1230,7 +1202,7 @@ public:
 
   // Report on inconsistent conditional instances.
   // Returns true if any mismatches.
-  bool reportInconsistentConditionals(llvm::raw_ostream &OS) {
+  bool reportInconsistentConditionals(llvm::raw_ostream &OS) override {
     bool ReturnValue = false;
     // Walk all the conditional trackers in the map.
     for (ConditionalExpansionMapIter I = ConditionalExpansions.begin(),
@@ -1296,6 +1268,9 @@ public:
   }
 
 private:
+  llvm::SmallVector<std::string, 32> HeaderList;
+  // Only do extern, namespace check for headers in HeaderList.
+  bool BlockCheckHeaderListOnly;
   llvm::StringPool Strings;
   std::vector<StringHandle> HeaderPaths;
   std::vector<HeaderHandle> HeaderStack;
@@ -1308,14 +1283,18 @@ private:
   bool InNestedHeader;
 };
 
+} // namespace
+
 // PreprocessorTracker functions.
 
 // PreprocessorTracker desctructor.
 PreprocessorTracker::~PreprocessorTracker() {}
 
 // Create instance of PreprocessorTracker.
-PreprocessorTracker *PreprocessorTracker::create() {
-  return new PreprocessorTrackerImpl();
+PreprocessorTracker *PreprocessorTracker::create(
+    llvm::SmallVector<std::string, 32> &Headers,
+    bool DoBlockCheckHeaderListOnly) {
+  return new PreprocessorTrackerImpl(Headers, DoBlockCheckHeaderListOnly);
 }
 
 // Preprocessor callbacks for modularize.
@@ -1356,7 +1335,7 @@ void PreprocessorCallbacks::FileChanged(
 
 // Handle macro expansion.
 void PreprocessorCallbacks::MacroExpands(const clang::Token &MacroNameTok,
-                                         const clang::MacroDirective *MD,
+                                         const clang::MacroDefinition &MD,
                                          clang::SourceRange Range,
                                          const clang::MacroArgs *Args) {
   clang::SourceLocation Loc = Range.getBegin();
@@ -1364,7 +1343,7 @@ void PreprocessorCallbacks::MacroExpands(const clang::Token &MacroNameTok,
   if (!Loc.isFileID())
     return;
   clang::IdentifierInfo *II = MacroNameTok.getIdentifierInfo();
-  const clang::MacroInfo *MI = PP.getMacroInfo(II);
+  const clang::MacroInfo *MI = MD.getMacroInfo();
   std::string MacroName = II->getName().str();
   std::string Unexpanded(getMacroUnexpandedString(Range, PP, MacroName, MI));
   std::string Expanded(getMacroExpandedString(PP, MacroName, MI, Args));
@@ -1374,11 +1353,11 @@ void PreprocessorCallbacks::MacroExpands(const clang::Token &MacroNameTok,
 }
 
 void PreprocessorCallbacks::Defined(const clang::Token &MacroNameTok,
-                                    const clang::MacroDirective *MD,
+                                    const clang::MacroDefinition &MD,
                                     clang::SourceRange Range) {
   clang::SourceLocation Loc(Range.getBegin());
   clang::IdentifierInfo *II = MacroNameTok.getIdentifierInfo();
-  const clang::MacroInfo *MI = PP.getMacroInfo(II);
+  const clang::MacroInfo *MI = MD.getMacroInfo();
   std::string MacroName = II->getName().str();
   std::string Unexpanded(getSourceString(PP, Range));
   PPTracker.addMacroExpansionInstance(
@@ -1408,7 +1387,7 @@ void PreprocessorCallbacks::Elif(clang::SourceLocation Loc,
 
 void PreprocessorCallbacks::Ifdef(clang::SourceLocation Loc,
                                   const clang::Token &MacroNameTok,
-                                  const clang::MacroDirective *MD) {
+                                  const clang::MacroDefinition &MD) {
   clang::PPCallbacks::ConditionValueKind IsDefined =
     (MD ? clang::PPCallbacks::CVK_True : clang::PPCallbacks::CVK_False );
   PPTracker.addConditionalExpansionInstance(
@@ -1419,7 +1398,7 @@ void PreprocessorCallbacks::Ifdef(clang::SourceLocation Loc,
 
 void PreprocessorCallbacks::Ifndef(clang::SourceLocation Loc,
                                    const clang::Token &MacroNameTok,
-                                   const clang::MacroDirective *MD) {
+                                   const clang::MacroDefinition &MD) {
   clang::PPCallbacks::ConditionValueKind IsNotDefined =
     (!MD ? clang::PPCallbacks::CVK_True : clang::PPCallbacks::CVK_False );
   PPTracker.addConditionalExpansionInstance(
